package Security::db;

use Exporter;
use DB_File;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() $db &verify() &print() &destroy() );

our ($db, %checksum, %badchecksums, %missingchecksumfiles);

sub init {
	print ( STDERR  "In Security::db::init()\n") if ( $main::opts{'d'} );

	$db=tie %checksum, 'DB_File', "$main::CONFDIR/.checksum.db", 
		O_CREAT|O_RDWR, 00600, $DB_HASH;

	&main::read_config('checksumfiles');
}

sub verify {
	print ( STDERR  "In Security::db::verify()\n") if ( $main::opts{'d'} );

	my ($file) = @_;
	my ($currsum);

	grep (/^$main::format$/ && ($currsum=$2), &main::SHA1($file));
	if ( defined $checksum{$file} && $currsum ne $checksum{$file} ) {
		$badchecksums{$file} = "old ($checksum{$file}) new ($currsum)";
	}
	$checksum{$file} = $currsum;
}

sub check {
	print ( STDERR  "In Security::db::check()\n") if ( $main::opts{'d'} );

	foreach my $file ( @main::checksumfiles ) {
		if ( -f $file ) {
			&verify($file);
		} else {
			$missingchecksumfiles{$file} = 1;
		}
	}
}

sub print {
	print ( STDERR  "In Security::db::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking checksum mismatch:\n", %badchecksums);
	&main::print_data ("\n\nMissing checksum files:\n", %missingchecksumfiles);
}

sub destroy {
	print ( STDERR  "In Security::db::destroy()\n") if ( $main::opts{'d'} );

	undef $db;
	untie %checksum;
}

1;
