package Security::sudo;

use Security::find;
use Exporter;
use POSIX;
use Security::db;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &check() &print() );

our (%sudoworld, %sudogroup, %sudoowner, %sudomissing, %groups);

sub init {
	print ( STDERR  "In Security::sudo::init()\n") if ( $main::opts{'d'} );

	&main::read_config('sudofile');
	&main::read_config('nosudoowner');
}

sub _get_group_members () {
	my ($gid) = @_;
	my ($nm, $pw, $u, $g);

	my (@gr) = getgrgid($gid);
	map { $groups{$_} = 1; } split /\s/o, $gr[3];

	# Now find the accounts that have this group as the primary gid
	setpwent;
	while (($nm, $pw, $u, $g) = getpwent()) {
		$groups{$nm} = 1
			if ($g eq $gid);
	}
	endpwent;

	return keys %groups;
}

sub check {
	print ( STDERR  "In Security::sudo::check()\n") if ( $main::opts{'d'} );

	my ($currsum, $cmds, $cmd, $who);
	open (FP, $main::sudofile ) || die "Could not read sudoers file ($main::sudofile)\n";
	while (<FP>) {
		chomp;
		if ( ! /^Default/o ) {
			my (@a) = split;

			# Now find the command.
			if ( exists $a[1] && defined $a[1] ) {
				grep (/:(.*)$/o && ($cmd=$1), $a[1]);
				grep (/=\((.*)\)/o && ($who=$1), $a[1]);

				# Now expand any wildcarding in the $cmd
				if ( defined $cmd && length $cmd && defined $who && length $who ) {
					foreach my $cmd2 ( glob $cmd ) {
						push @{$cmds->{$cmd2}->{who}}, $who;
					}
				}
			}
		}
	}
	close FP;

	# Clean up the who list to have only uniq names
	foreach $cmd ( keys %$cmds ) {
		my (@a, $last);
		foreach $who ( sort @{$cmds->{$cmd}->{who}} ) {
			push @a, $who
				if ( !defined $last || $last ne $who );
			$last = $who;
		}
		@{$cmds->{$cmd}->{who}} = @a;
	}

	# Now get mode/uid/gid info for each program 
	# and check who can write to the file.
	foreach $cmd ( keys %$cmds ) {
 		if ( ! -f $cmd ) {
			$sudomissing{$cmd} = 1;		
 		} else {
			&Security::db::verify($cmd);

			my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = 
				&Security::find::get_stat($cmd, 'stat');

			# If the file is world writeable, then flag it
			$sudoworld{$cmd} = 1
				if ( $mode & &S_IWOTH() );

			# Now check if it is group writeable, and who is in the group
			if ( $mode & &S_IWGRP() ) {
				my @list = &_get_group_members ($gid);
				$who = getpwuid($uid);

				# If the owner is NOT in the list of group members, then
				# complain about this.
				if (!grep (/^$who$/, @list)) {
					$sudogroup{$cmd} = 1;
				}
			}

			# Lastly, check to make sure if the file is user writeable
			# and who owns the file (exclude files owned by root or
			# bin (uid=$binUID).  This can be ignore if nosudoowner is
			# set in the config file.
			if ( ${main::nosudoowner[0]} ne 'yes' && $mode & &S_IWUSR() && $uid != 0 && $uid != $main::binuid ) {
				$who = getpwuid($uid);
				if (! grep (/^${who}$/, @{$cmds->{$cmd}->{who}})) {
					$sudoowner{$cmd} = 1;
				}
			}
		}
	}
}

sub print {
	print ( STDERR  "In Security::sudo::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking sudo missing commands:\n", %sudomissing);
	&main::print_data ("\n\nchecking for world writable sudo commands:\n", %sudoworld);
	&main::print_data ("\n\nchecking for group writable sudo commands:\n", %sudogroup);
	&main::print_data ("\n\nchecking for owner mismatch sudo commands:\n", %sudoowner);
}

1;
