package Security::suid;

use Security::find;
use Exporter;
use POSIX;
use Security::db;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &print() );

our (%suidperms);

sub init {
	print ( STDERR  "In Security::suid::init()\n") if ( $main::opts{'d'} );

	&Security::find::add_wanted (\&suid_wanted);
}

# Check the permissions and maybe the checksum of all suid/sgid files.
# From the FreeBSD /etc/security script
sub suid_wanted {
	my ($currsum);

	if ( -x $File::Find::name && -f $File::Find::name &&
		! -l $File::Find::name &&
		( -u $File::Find::name || -g $File::Find::name)) {
		my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = 
			&Security::find::get_stat($File::Find::name, 'lstat');

		&Security::db::verify($File::Find::name);

		# If the file is world writeable, then flag this case too
		$suidperms{$File::Find::name} = 1
			if ( $mode & &S_IWOTH() );
	}
}

sub print {
	print ( STDERR  "In Security::suid::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking for world writable suid/sgid programs:\n", %suidperms);
}

1;
